#
# krb5.conf(5) - configuration file for Kerberos 5
# $FreeBSD$
#
<%
        import logging
        logger = logging.getLogger(__name__)
        from middlewared.utils import filter_list
        from middlewared.plugins.kerberos import KRB_AppDefaults, KRB_LibDefaults

        def parse_defaults(section_name, section_conf, db_def=None):
            default_section = "krb5_main"
            if section_name == "appdefault":
                supported_options = [i.value[0] for i in KRB_AppDefaults]
            elif section_name == "libdefault":
                supported_options = [i.value[0] for i in KRB_LibDefaults]

            db_def_lines = db_def.splitlines()
            for line in db_def_lines:
                ws_lines = line.split()
                if len(ws_lines) < 1:
                    continue
                if ws_lines[-1] == "{":
                    default_section = ws_lines[0]
                elif ws_lines[0] == "}":
                    default_section = "krb5_main"
                    continue
                elif ws_lines[0] in supported_options:
                    binding = line.split("=")
                    if default_section in section_conf:
                        section_conf[default_section].update({binding[0].strip(): binding[1].strip()})
                    else:
                        section_conf.update({default_section: {binding[0].strip(): binding[1].strip()}})
                else:
                    continue

            return section_conf

        db = {}
        db_realms = []
        environment_is_kerberized = False
        db['ad'] = middleware.call_sync('activedirectory.config')
        if not db['ad']['enable']:
            db['ldap'] = middleware.call_sync('ldap.config')

        db['krb_aux'] = middleware.call_sync('kerberos.config')
        db_realms = middleware.call_sync('kerberos.realm.query')
        krb_default_realm = None

        appdefaults = {'pam': {
            KRB_AppDefaults.FORWARDABLE.parm(): 'true',
            KRB_AppDefaults.TICKET_LIFETIME.parm(): '86400',
            KRB_AppDefaults.RENEW_LIFETIME.parm(): '86400'
        }}

        libdefaults = {'krb5_main': {
            KRB_LibDefaults.DNS_LOOKUP_REALM.parm(): 'true',
            KRB_LibDefaults.DNS_LOOKUP_KDC.parm(): 'true',
            KRB_LibDefaults.TICKET_LIFETIME.parm(): '24h',
            KRB_LibDefaults.CLOCKSKEW.parm(): '300',
            KRB_LibDefaults.FORWARDABLE.parm(): 'true'
        }}

        """
            Active Directory environments are always kerberized. This means
            we make best effort to make a valid krb5.conf and update set correct
            values in the configuration database.

            On the other hand, not all LDAP environments are kerberized so we cannot guess the correct
            realm in this case.
        """
        if db['ad']['enable'] and db['ad']['kerberos_realm']:
            environment_is_kerberized = True
            c_realm = db['ad']['kerberos_realm']
            krb_default_realm = (filter_list(db_realms, [('id', '=', c_realm)]))[0]['realm']

        elif db['ad']['enable']:
            environment_is_kerberized = True
            krb_default_realm = db['ad']['domainname']
            if not krb_default_realm:
                # AD is enabled with a thoroughly invalid configuration. Disabled and refuse to generate
                # kerberos config.
                middleware.call_sync("activedirectory.direct_update", {"enable": False})
                raise FileShouldNotExist()

            if db_realms:
                db_realm_entry = next((item for item in db_realms if item['realm'] == krb_default_realm), None)
            else:
                db_realm_entry = None

            if db_realm_entry:
                logger.debug(f'Updating Active Directory configuration to use kerberos realm: {db_realm_entry}')
                middleware.call_sync('activedirectory.direct_update', {'kerberos_realm': db_realm_entry['id']})
                db_realms.append({
                    'realm': db_realm_entry['realm'],
                    'kdc': [],
                    'admin_server': [],
                    'kpasswd_server': []
                })
            else:
                logger.debug(f'Generating kerberos realm entry for {krb_default_realm}')
                id = middleware.call_sync(
                    'kerberos.realm.direct_create',
                    {'realm': ad['domainname'].upper(), 'kdc': '', 'admin_server': '', 'kpasswd_server': ''}
                )
                middleware.call_sync('activedirectory.direct_update', {'kerberos_realm': id})
                db_realms.append({
                    'realm': krb_default_realm,
                    'kdc': [],
                    'admin_server': [],
                    'kpasswd_server': []
                })

        elif db['ldap']['enable'] and db['ldap']['kerberos_realm']:
            environment_is_kerberized = True
            ldap_realm = filter_list(db_realms, [('id', '=', db['ldap']['kerberos_realm'])])
            if ldap_realm:
                krb_default_realm = ldap_realm[0]['realm']

        if krb_default_realm:
            libdefaults['krb5_main'].update({KRB_LibDefaults.DEFAULT_REALM: krb_default_realm})

        parsed_appdefaults = parse_defaults("appdefault", appdefaults, db_def=db['krb_aux']['appdefaults_aux'])
        parsed_libdefaults = parse_defaults("libdefault", libdefaults, db_def=db['krb_aux']['libdefaults_aux'])

        if not db_realms:
            raise FileShouldNotExist()
%>
[appdefaults]
% for section_name, section in parsed_appdefaults.items():
            % if section_name == "krb5_main":
            % for binding, value in section.items():
            ${binding} = ${value}
            % endfor
            % else:
            ${section_name} = {
            % for binding, value in section.items():
                   ${binding} = ${value}
            % endfor
            }
            % endif
% endfor

[libdefaults]
% for section_name, section in parsed_libdefaults.items():
            % for binding, value in section.items():
            ${binding} = ${value}
            % endfor
% endfor

[domain_realm]
% for realm in db_realms:
            ${realm["realm"].lower()} = ${realm["realm"]}
            .${realm["realm"].lower()} = ${realm["realm"]}
            ${realm["realm"].upper()} = ${realm["realm"]}
            .${realm["realm"].upper()} = ${realm["realm"]}
% endfor

[realms]
% for realm in db_realms:
            ${f'{realm["realm"]}'} = {
                   default_domain = ${realm["realm"]}
                % for k in realm["kdc"]:
                   kdc = ${k}
                % endfor
                % if realm["admin_server"]:
                   admin_server = ${' '.join(realm["admin_server"])}
                % endif
                % if realm["kpasswd_server"]:
                   kpasswd_server = ${' '.join(realm["kpasswd_server"])}
                % endif
            }
% endfor

[logging]
            default = SYSLOG:INFO:LOCAL7
